'use strict';

const debuglog = __debug.bind(null, __filename);

const _ = require('lodash');
const path = require('path');


const configUtil = require('../utils/config-util');
const manifest = require('../manifest');
const oauth = require('../routes/oauth2');


function constructV2Response(req) {
  const currentProduct = req.query.product;

  if (!manifest.product[currentProduct]) {
    return [];
  }

  const locations = JSON.parse(JSON.stringify(manifest.product[currentProduct].location));

  function convertURLs(locations, oauth) {
    _.forEach(locations, (value) => {
      value.url = 'http://localhost:10001/iframe/' + (oauth || value.url);
    });
  }

  const appDetails = {
    id: 1,
    version_id: 1,
    configs: configUtil.getValuesForLocalTesting(),
    displayName: process.cwd().split(path.sep).pop(),
    placeholders: locations
  };

  if (_.includes(manifest.features, 'oauth') && !_.has(oauth.fetchCredentials(req), 'access_token')) {
    convertURLs(appDetails.placeholders, 'oauth/local-testing-oauth-msg.html');
  } else {
    convertURLs(appDetails.placeholders);
  }

  return [ appDetails ];
}

function iframeAPIHandlerV2 (req, res) {
  const appDetails = constructV2Response(req);

  debuglog(`Responding to product with ${JSON.stringify(appDetails)}`);

  res.send(appDetails);
}

function iframeAPIHandler (req, res) {
  const appDetails = {
    configs: configUtil.getNonSecureValues(),
    displayName: process.cwd().split(path.sep).pop(),
    product: manifest.product,
    features: manifest.features
  };

  if (_.includes(manifest.features, 'oauth') && oauth.isNotAgent()) {
    let isAuthorized = false;

    if (_.has(oauth.fetchCredentials(req), 'access_token')) {
      isAuthorized = true;
    }
    _.extend(appDetails, { isAuthorized: isAuthorized });
  }

  appDetails.v2 = constructV2Response(req);

  debuglog(`Responding to product with ${JSON.stringify(appDetails)}`);

  res.send(appDetails);
}

module.exports = {
  run: (app) => {
    app.get('/iframe/api', iframeAPIHandler);
    app.get('/iframe/api/v2', iframeAPIHandlerV2);
  }
};
