'use strict';

/* eslint-disable no-underscore-dangle */

const _ = require('lodash');

const DataStore = require('./data-store');
const eh = require('./err');
const fileUtil = require('./file-util');
const nsUtil = require('./ns-resolver');

const jsonFile = 'iparams.json';
const htmlFile = 'iparams.html';
const oauthConfigFile = 'oauth_config.json';
const testDataFile = 'iparam_test_data.json';

const storage = new DataStore({});

function getJSONContent(fileName) {
  const data = fileUtil.readFile(`${process.cwd()}/config/${fileName}`);

  try {
    return JSON.parse(data);
  }
  catch (e) {
    console.log(`${fileName}:`);
    eh.error(new Error(e));
  }
}

function getIparams(onlyNonSecure) {
  const HTML_FILE = `${process.cwd()}/config/iparams.html`;
  const JSON_FILE = `${process.cwd()}/config/iparams.json`;
  let iParams = {};

  if (fileUtil.fileExists(HTML_FILE)) {
    iParams = storage.fetch(nsUtil.getInternalNamespace('custom_iparams')) || {};

    if (onlyNonSecure) {
      if (iParams.__meta) {
        const secure = iParams.__meta.secure;

        if (secure && secure.length) {
          iParams = _.omit(iParams, secure);
        }
      }
    }
    else {
      // Remove __meta tag.
      delete iParams.__meta;
    }
  }
  else if (fileUtil.fileExists(JSON_FILE)) {
    iParams = getJSONContent(testDataFile);

    if (onlyNonSecure) {
      const iParamsConfig = getJSONContent(jsonFile);
      const secureKeys = _.keys(iParams).filter(
        (prop) => iParamsConfig[prop] && iParamsConfig[prop].secure);

      iParams = _.omit(iParams, secureKeys);
      // Add __meta tag to show relevant error message while retrieving secure iParams in UI.
      iParams.__meta = { secure: secureKeys };
    }
  }
  return iParams;
}

module.exports = {
  getConfig() {
    return getJSONContent(jsonFile);
  },

  getValuesForLocalTesting() {
    return getIparams();
  },

  getNonSecureValues() {
    return getIparams(true);
  },

  getHTMLContent() {
    return fileUtil.readFile(`${process.cwd()}/config/${htmlFile}`);
  },

  getOAuthIparam() {
    return getJSONContent(oauthConfigFile);
  }
};
