'use strict';

const fs = require('fs');

const debuglog = __debug.bind(null, __filename);

const DEFAULT_PATH = `${process.cwd()}/.fdk/`;
const DATASTORE = require('./data-store');
var reportUtil = require('./report-util');
var fileUtil = require('./file-util');

const store = new DATASTORE({location: DEFAULT_PATH, file: 'metrics'});

function readMetricFile() {
  //Reading metrics from localstore metrics file
  var metricData = {};

  if (fileUtil.fileExists(`${DEFAULT_PATH}metrics`)) {
    metricData = store.readLocalStore();
  }
  return metricData;
}

module.exports = {
  store(key, value){
    debuglog(`Metric: version ${key}, ${value}`);
    var previous_data = store.fetch(key);

    if (key !== undefined) {
      if (previous_data === undefined) {
        previous_data = [];
      }
      previous_data.push(value);
      //Store value as array of metric data:-> Key: Related data (e.g. : {version: [version info]})
      store.store(key, previous_data);
    }
  },

  fetch(key) {
    return store.fetch(key);
  },

  addMetrics(callback){
    var filePath = `${process.cwd()}/.report.json`;

    debuglog(`writing metrics to  ${filePath}`);
    var metricData = readMetricFile();

    //storing it in .report.json
    reportUtil.set('metricData', metricData);
    //as reportutil was writing after exit of process but we need before app exits
    fs.writeFile(filePath, JSON.stringify(reportUtil.get(undefined)), callback);
  },

  removeMetrics() {
    var file = `${DEFAULT_PATH}metrics`;

    fileUtil.deleteFile(file);
    reportUtil.set('metricData', {});

  }
};
