'use strict';

const tmp = require('tmp');
const fs = require('fs-extra');
const request = require('supertest');

const httpUtil = require('../lib/utils/http-util');

describe('SMI Test', () => {
  let server, projDir, payload, headers;

  before(() => {
    projDir = tmp.dirSync({prefix: 'freshapps_sdk'});
    require(__dirname + '/../lib/cli/init').run(projDir.name, 'your_first_serverless_app', 'freshdesk');
    process.chdir(projDir.name);
    require(__dirname + '/../lib/manifest').reload();
    payload = {
      methodName: 'callRemote',
      methodParams: { url: 'https://www.google.co.in' },
      action: 'invoke'
    };
    headers = {
      'content-type': 'application/json',
      'MKP-EXTNID': '101',
      'MKP-VERSIONID': '101',
      'MKP-ROUTE': 'smi'
    };
    delete require.cache;
  });

  afterEach(() => {
    server.close();
  });

  it ('should invoke the function in the server script', (done) => {
    const destFile = `${projDir.name}/server/server.js`;
    const srcFile = `${__dirname}/../test-res/backend_server.js`;

    fs.copySync(srcFile, destFile);
    server = require(__dirname + '/../lib/cli/run').run();
    request(server)
      .post('/dprouter')
      .send(payload)
      .set(headers)
      .expect(httpUtil.status.ok, done);
  });

});